@@ -6,14 +6,14 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: simplate.cpp 32 2007-09-08 00:18:57Z kazuhiro $ */
+/* $Id: simplate.cpp 49 2008-07-18 01:23:19Z kazuhiro $ */
 
 /**
  * @file simplate.cpp
  * @brief Simplate class implementation
  * @author Kazuhiro IIzuka
  * @author Shimizu
- * @version $Id: simplate.cpp 32 2007-09-08 00:18:57Z kazuhiro $
+ * @version $Id: simplate.cpp 49 2008-07-18 01:23:19Z kazuhiro $
  * Copyright (C) Kazuhiro IIzuka
  */
 
@@ -96,7 +96,7 @@ static int le_simplate;
 #define COMPILE_CHECK "compile_check"
 #define FORCE_COMPILE "force_compile"
 #define LAZY_CHECK "lazy_check"
-#define VERSION "0.3.7"
+#define VERSION "0.4.1"
 #define CACHE_LIFETIME "cache_lifetime"
 #define CACHING "caching"
 
@@ -203,6 +203,63 @@ static void simplate_objects_clone(void *object,void **object_clone TSRMLS_DC)
 }
 #endif // 0
 
+static string get_include_filename(zval *obj,string template_dir,string filename TSRMLS_DC)
+{
+	string included_filename;
+	if( filename.find("$")!=string::npos ){
+		bool find_filename = false;
+		string include_variable = "";
+		for(int i=0;i<filename.length();i++){
+			if( filename[i]=='$' ){
+				find_filename = true;
+			}else if( filename[i]=='.' ){
+				break;
+			}
+			if( find_filename ){
+				include_variable+=filename[i];
+			}
+//cerr << "[B2]include_variable=" << include_variable << endl;
+		}
+		include_variable.erase(0,1);
+#ifdef ZEND_ENGINE_2
+		zval *_tpl_vars=zend_read_property(Z_OBJCE_P(obj),obj,"_tpl_vars",strlen("_tpl_vars"),1 TSRMLS_CC);
+#else
+		zval *_tpl_vars;
+		if (zend_hash_find(Z_OBJPROP_P(obj), "_tpl_vars", sizeof("_tpl_vars"), (void**)&_tpl_vars) != SUCCESS) {
+// TODO:
+		}
+#endif // ZEND_ENGINE_2
+		zval **zinclude_file;
+		string new_filename;
+		if( zend_hash_find(Z_ARRVAL_P(_tpl_vars),const_cast<char*>(include_variable.c_str()),include_variable.length()+1,(void**)&zinclude_file) == SUCCESS && Z_TYPE_PP(zinclude_file)==IS_STRING ){
+//cerr << include_variable << " -> " << Z_STRVAL_PP(zinclude_file) << endl;
+			new_filename = filename.replace(filename.find("$"),include_variable.length()+1,Z_STRVAL_PP(zinclude_file));
+		}else{
+			// TODO:
+			zend_error(E_ERROR,"can't find include file.");
+		}
+		if( new_filename[0] == DEFAULT_SLASH ){
+			included_filename = new_filename;
+		}else{
+			if( included_filename[0] == DEFAULT_SLASH ){
+				included_filename = new_filename;
+			}else{
+				included_filename = template_dir;
+				included_filename+= DEFAULT_SLASH;
+				included_filename+= new_filename;
+			}
+
+		}
+	}else if( filename[0] == DEFAULT_SLASH ){
+		// <{include file="/path/to/file.tpl"}>
+		included_filename=filename;
+	}else{
+		included_filename=template_dir;
+		included_filename+=DEFAULT_SLASH;
+		included_filename+=filename;
+	}
+	return included_filename;
+}
 
 static string _readfile(const char*filename TSRMLS_DC)
 {
@@ -334,9 +391,9 @@ static string parse_variable(const char *variable)
 	const char *start_tag,*end_tag;
 	string new_variable;
 
-	if (variable[0] != '$' || strcmp(variable,"$this")==0 ) {
+	if (variable[0] != '$' || strncmp(variable,"$this",5)==0 ) {
 		new_variable=variable;
-		str_replace(new_variable,"[","[$");
+//		str_replace(new_variable,"[","[$");
 //cerr << variable << " =>" << new_variable << endl;
 		return new_variable;
 	}
@@ -346,6 +403,7 @@ static string parse_variable(const char *variable)
 		// $hoge[i] -> _tpl_vars["hoge"][$i]
 		// $hoge[i].foo.bar -> _tpl_vars["hoge"][$i]['foo']['bar']
 		// $hoge[i].huga[j].foo -> _tpl_vars['hoge'][$i]['huga'][$j]['foo']
+		// $hoge[i]->huga[j]->foo -> _tpl_vars['hoge'][$i]->huga[$j]->foo
 //cerr << "variable=" << variable << " -> ";
 		int i=0;
 		while( variable[i] ){
@@ -367,6 +425,10 @@ static string parse_variable(const char *variable)
 				i+=get_identifier(variable,i,new_variable);
 				new_variable+="]";
 				i++;
+			}else if( i>0 && variable[i] == '>' && variable[i-1] == '-'){
+				new_variable+="->";
+				i++;
+				i+=get_identifier(variable,i,new_variable);
 			}else{
 				i++;
 			}
@@ -395,10 +457,24 @@ static string parse_variable(const char *variable)
 			new_variable+="$this->_tpl_vars";
 			for (vector<string>::iterator i = elements.begin();i != elements.end(); ++i) {
 				new_variable+="[\"";
-				new_variable+=*i;
-				new_variable+="\"]";
+
+				size_t pos=0;
+				if( (pos=(*i).find("->"))!=string::npos ){
+					new_variable+=(*i).substr(0,pos);
+					new_variable+="\"]";
+					new_variable+=(*i).substr(pos);
+				}else{
+					new_variable+=*i;
+					new_variable+="\"]";
+				}
 			}
 		}
+	}else if(strstr(variable,"->")){
+		string temp = variable;
+		new_variable+="$this->_tpl_vars['";
+		new_variable+=temp.substr(1,temp.find("->")-1);
+		new_variable+="']";
+		new_variable+=temp.substr(temp.find("->"));
 	}else{
 		// normal variable
 		// {$key} -><?php $this->_tpl_vars['key']; ?>
@@ -535,8 +611,7 @@ static inline bool _is_compiled(zval *obj, const char *template_dir, const char
 				tag_start_pos=pos;
 				element_start_pos=tag_start_pos+strlen(left_delimiter);
 				element_end_pos=file_content.find(right_delimiter,tag_start_pos);
-				tag_end_pos=element_end_pos+strlen(right_delimiter);
-				if (tag_end_pos == string::npos) {
+				if (element_end_pos == string::npos) {
 					error="No closed delimiter:`";
 					error+=right_delimiter;
 					error+="' in ";
@@ -544,6 +619,7 @@ static inline bool _is_compiled(zval *obj, const char *template_dir, const char
 					zend_error(E_ERROR,error.c_str());
 					return false;
 				}
+				tag_end_pos=element_end_pos+strlen(right_delimiter);
 				item.assign(file_content,element_start_pos,element_end_pos-element_start_pos);
 				item=trim(item.c_str());
 				if (item[0]=='$') { pos++;continue; }
@@ -551,18 +627,8 @@ static inline bool _is_compiled(zval *obj, const char *template_dir, const char
 				if (item.substr(0,7)=="include" ){
 //cerr << "item=`" << item << "'" << endl;
 					string filename=get_element(item.c_str(),"file");
-					string included_filename;
+					string included_filename = get_include_filename(obj,template_dir,filename TSRMLS_CC);
 					// <{include file="$hoge"}> includes $smarty->_tpl_vars['hoge'] named file.
-					if (filename.find("$") != string::npos) {
-						// TODO:not implement
-					}else if (filename[0] == DEFAULT_SLASH /*'/'*/) {
-						// <{include file="/tmp/hoge.tpl"}>
-						included_filename=filename;
-					}else{
-						included_filename=template_dir;
-						included_filename+=DEFAULT_SLASH;
-						included_filename+=filename;
-					}
 
 					struct stat is;
 					// Check included template file timestamp
@@ -648,7 +714,7 @@ void read_parse_template(INTERNAL_FUNCTION_PARAMETERS,char **fullfile_name,int m
   zval **temp_zval;
 //  char *k=0;
 //  ulong i=0;
-  // PHP4ǤϡΤ褦ˤʤvar_dump($this)ʤä
+  // PHP4では、次のようにしないとvar_dump($this)が取れなかった。
   // $this=&$smarty;
 //  zend_hash_update(&EG(symbol_table),"this",strlen("this")+1,&obj,sizeof(zval*),NULL);
 
@@ -913,8 +979,7 @@ compile:
 			tag_start_pos=pos;
 			element_start_pos=tag_start_pos+strlen(left_delimiter);
 			element_end_pos=compiled_file_content.find(right_delimiter,tag_start_pos);
-			tag_end_pos=element_end_pos+strlen(right_delimiter);
-			if (tag_end_pos == string::npos) {
+			if (element_end_pos == string::npos) {
 				error="No closed delimiter:`";
 				error+=right_delimiter;
 				error+="' in ";
@@ -922,6 +987,8 @@ compile:
 				zend_error(E_ERROR,error.c_str());
 				return;
 			}
+			tag_end_pos=element_end_pos+strlen(right_delimiter);
+
 			item.assign(compiled_file_content,element_start_pos,element_end_pos-element_start_pos);
 			item=trim(item.c_str());
 			string variable;
@@ -938,7 +1005,7 @@ cond_var:
 //cerr << "tag_start_pos=" << tag_start_pos << endl;
 				while (element_start_pos + i < element_end_pos) {
 					if (compiled_file_content[element_start_pos + i] == '+'
-					 || compiled_file_content[element_start_pos + i] == '-'
+					 || (compiled_file_content[element_start_pos + i] == '-' && compiled_file_content.at(element_start_pos+i+1)!='>')
 					 || compiled_file_content[element_start_pos + i] == '*'
 					 || compiled_file_content[element_start_pos + i] == '/'
 					 || compiled_file_content[element_start_pos + i] == '%'
@@ -954,8 +1021,12 @@ cond_var:
 					 || compiled_file_content[element_start_pos + i] == ' '
 					) {
 
-						new_variable = parse_variable(variable.c_str());
+						if( variable!="$" ){
+							new_variable = parse_variable(variable.c_str());
 //cerr << "val1=" << variable << " -> " << new_variable << endl;
+						}else{
+							new_variable = variable;
+						}
 						new_condition+=new_variable;
 						new_condition+=compiled_file_content[element_start_pos+i];
 						variable="";
@@ -1001,18 +1072,19 @@ cond_if:
 					i=0;
 					while (i < condition.length()) {
 						if (condition[i]=='+'
-						  ||condition[i]=='-'
+						  ||(condition[i]=='-' && condition.at(i+1)!='>')
 						  ||condition[i]=='*'
 						  ||condition[i]=='/'
 						  ||condition[i]=='%'
 						  ||condition[i]=='<'
-						  ||condition[i]=='>'
+						  ||(condition[i]=='>' && condition.at(i-1)!='-')
 						  ||condition[i]=='='
 						  ||condition[i]=='!'
 						  ||condition[i]=='|'
 						  ||condition[i]=='&'
 						  ||condition[i]=='('
 						  ||condition[i]==')'
+						  ||condition[i]==','
 						  ||condition[i]==' '
 						) {
 							new_variable = parse_variable(variable.c_str());
@@ -1212,57 +1284,8 @@ cond_if:
 					pos+=strlen("<?php }} ?>")-1; // 20060527
 				}else if (command_tag == "include") {
 					string filename=get_element(item.c_str(),"file");
-					string included_filename;
-					// <{include file="var_$hoge"}> includes var_$smarty->_tpl_vars['hoge'] named file.
-					if ((i = filename.find("$")) != string::npos) {
-//cerr << "filename=" << filename << endl;
-						bool find_filename=false;
-						string include_variable = "";
-						for(int j=0;j<filename.length();j++){
-							if( filename[j]=='$' ){
-								find_filename=true;
-							}else if( filename[j]=='.' ){
-								break;
-							}
-							if( find_filename ){
-								include_variable+=filename[j];
-							}
-						}
-						include_variable.erase(0,1);
+					string included_filename = get_include_filename(obj,template_dir,filename TSRMLS_CC);
 
-//						_tpl_vars[filename]
-#ifdef ZEND_ENGINE_2
-						zval *_tpl_vars=zend_read_property(Z_OBJCE_P(obj),obj,"_tpl_vars",strlen("_tpl_vars"),1 TSRMLS_CC);
-#else
-						zval *_tpl_vars;
-						if (zend_hash_find(Z_OBJPROP_P(obj), "_tpl_vars", sizeof("_tpl_vars"), (void**)&_tpl_vars) != SUCCESS) {
-// TODO:
-						}
-#endif // ZEND_ENGINE_2
-						zval **include_file;
-						string new_filename;
-						if( zend_hash_find(Z_ARRVAL_P(_tpl_vars),const_cast<char*>(include_variable.c_str()),include_variable.length()+1,(void**)&include_file) == SUCCESS && Z_TYPE_PP(include_file)==IS_STRING ){
-//cerr << include_variable << " -> " << Z_STRVAL_PP(include_file) << endl;
-							new_filename = filename.replace(filename.find("$"),include_variable.length()+1,Z_STRVAL_PP(include_file));
-
-						}else{
-// TODO:
-						}
-						if( included_filename[0] == DEFAULT_SLASH ){
-							included_filename = new_filename;
-						}else{
-							included_filename = template_dir;
-							included_filename+=DEFAULT_SLASH;
-							included_filename+=new_filename;
-						}
-					}else if (filename[0] == DEFAULT_SLASH /*'/'*/) {
-						// <{include file="/tmp/hoge.tpl"}>
-						included_filename=filename;
-					}else{
-						included_filename=template_dir;
-						included_filename+=DEFAULT_SLASH;
-						included_filename+=filename;
-					}
 					new_condition = _readfile(included_filename.c_str() TSRMLS_CC);
 					if (new_condition.find(filename) != string::npos) {
 						error="Found recursive include:";
@@ -1377,7 +1400,7 @@ cerr << "FILE:[[" << compiled_file_content << "]]" << endl;
 			fp = VCWD_FOPEN(full_compile_filename.c_str(),"wb");
 			if( fp == NULL ){
 				// error handling
-				zend_error(E_ERROR,"fail to open: %s",full_compile_filename.c_str());
+				zend_error(E_ERROR,"fail to write : %s",full_compile_filename.c_str());
 				return;
 			}
 		}
